@@ -29,11 +29,12 @@
 #include "misc.h"
 #include "xdmcp.h"
 
-static const gchar RCSid[]="$Id: display.c,v 1.4 2001/01/10 18:55:11 jirka Exp $";
+static const gchar RCSid[]="$Id: display.c,v 1.6 2001/01/23 01:59:16 jirka Exp $";
 
 /* External vars */
 extern gint GdmXdmcp;
 extern gint sessions;
+extern gint pending;
 extern GSList *displays;
 
 
@@ -60,8 +61,9 @@ gdm_display_manage (GdmDisplay *d)
 	sigaddset (&mask, SIGCHLD);
 	sigprocmask (SIG_BLOCK, &mask, &omask); 
 
-	kill (d->slavepid, SIGINT);
-	waitpid (d->slavepid, 0, 0);
+	if (kill (d->slavepid, SIGINT) == 0)
+		waitpid (d->slavepid, 0, 0);
+	d->slavepid = 0;
 
 	sigprocmask (SIG_SETMASK, &omask, NULL);
     }
@@ -77,11 +79,15 @@ gdm_display_manage (GdmDisplay *d)
 	if (GdmXdmcp)
 	    gdm_xdmcp_close ();
 
-	if (d->type == TYPE_LOCAL)
+	if (d->type == TYPE_LOCAL) {
 	    gdm_slave_start (d);
-
-	if (d->type == TYPE_XDMCP && d->dispstat == XDMCP_MANAGED)
+	    /* if we ever return, bad things are happening */
+	    _exit (DISPLAY_ABORT);
+	} else if (d->type == TYPE_XDMCP && d->dispstat == XDMCP_MANAGED) {
 	    gdm_slave_start (d);
+	    /* we expect to return after the session finishes */
+	    _exit (DISPLAY_REMANAGE);
+	}
 
 	break;
 
@@ -112,12 +118,13 @@ gdm_display_unmanage (GdmDisplay *d)
     if (!d)
 	return;
 
-    gdm_debug ("gdm_display_unmanage: Stopping %s", d->name);
+    gdm_debug ("gdm_display_unmanage: Stopping %s (slave pid: %d)",
+	       d->name, (int)d->slavepid);
 
-    /* Kill slave and all its children */
-    if (d->slavepid) {
-	kill (-(d->slavepid), SIGTERM);
-	waitpid (d->slavepid, 0, 0);
+    /* Kill slave */
+    if (d->slavepid != 0) {
+	if (kill (d->slavepid, SIGTERM) == 0)
+		waitpid (d->slavepid, 0, 0);
 	d->slavepid = 0;
     }
     
@@ -145,6 +152,9 @@ gdm_display_dispose (GdmDisplay *d)
 	displays = g_slist_remove (displays, d);
 	sessions--;
 	d->type = -1;
+
+	if (d->dispstat == XDMCP_PENDING)
+		pending--;
     }
 
     if (d->name) {
