@@ -18,6 +18,7 @@
 
 #include <config.h>
 #include <gnome.h>
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -39,15 +40,15 @@
 #include "gdm.h"
 #include "filecheck.h"
 
-static const gchar RCSid[]="$Id: gdmlogin.c,v 1.16 2001/01/10 06:11:09 jirka Exp $";
+static const gchar RCSid[]="$Id: gdmlogin.c,v 1.19 2001/01/26 04:00:51 jirka Exp $";
 
-static gint  GdmBrowser;
-static gint  GdmDebug;
+static gboolean GdmBrowser;
+static gboolean GdmDebug;
 static gint  GdmIconMaxHeight;
 static gint  GdmIconMaxWidth;
-static gint  GdmQuiver;
+static gboolean GdmQuiver;
 static gint  GdmRelaxPerms;
-static gint  GdmSystemMenu;
+static gboolean GdmSystemMenu;
 static gint  GdmUserMaxFile;
 static gchar *GdmLogo;
 static gchar *GdmWelcome;
@@ -61,6 +62,10 @@ static gchar *GdmDefaultLocale;
 static gchar *GdmExclude;
 static gchar *GdmGlobalFaceDir;
 static gchar *GdmDefaultFace;
+static gboolean GdmLockPosition;
+static gboolean GdmSetPosition;
+static int GdmPositionX;
+static int GdmPositionY;
 
 static GtkWidget *login;
 static GtkWidget *label;
@@ -71,31 +76,31 @@ static GtkWidget *win;
 static GtkWidget *sessmenu;
 static GtkWidget *langmenu;
 static GdkWindow *rootwin;
-static GdkWindow *rootwin_overlay;
+static GdkWindow *rootwin_overlay = NULL;
 
 static GnomeIconList *browser;
 static GdkImlibImage *defface;
 
 /* Eew. Loads of global vars. It's hard to be event controlled while maintaining state */
-GSList *sessions = NULL;
-GSList *languages = NULL;
-GList *users = NULL;
-GSList *exclude = NULL;
+static GSList *sessions = NULL;
+static GSList *languages = NULL;
+static GList *users = NULL;
+static GSList *exclude = NULL;
 
-gchar *defsess=NULL;
-gchar *cursess=NULL;
-gchar *curlang=NULL;
-gchar *curuser=NULL;
-gchar *lastsess=NULL;
-gchar *lastlang=NULL;
-gchar *session=NULL;
-gchar *language=NULL;
+static gchar *defsess = NULL;
+static gchar *cursess = NULL;
+static gchar *curlang = NULL;
+static gchar *curuser = NULL;
+static gchar *lastsess = NULL;
+static gchar *lastlang = NULL;
+static gchar *session = NULL;
+static gchar *language = NULL;
 
-gboolean savesess;
-gboolean savelang;
-gint maxwidth;
+static gboolean savesess;
+static gboolean savelang;
+static gint maxwidth;
 
-pid_t backgroundpid = 0;
+static pid_t backgroundpid = 0;
 
 static void
 gdm_greeter_chld (int sig)
@@ -111,6 +116,7 @@ kill_background (void)
 {
 	if (backgroundpid != 0) {
 		kill (backgroundpid, SIGTERM);
+		backgroundpid = 0;
 	}
 }
 
@@ -121,78 +127,113 @@ gdm_login_done (int sig)
     _exit (DISPLAY_SUCCESS);
 }
 
-
-typedef struct _cursoroffset { gint x,y; } CursorOffset;
-
-
 static void
+set_screen_pos (GtkWidget *widget, int x, int y)
+{
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	if (x < 0)
+		x = 0;
+	if (y < 0)
+		y = 0;
+	if (x > gdk_screen_width () - widget->allocation.width)
+		x = gdk_screen_width () - widget->allocation.width;
+	if (y > gdk_screen_height () - widget->allocation.height)
+		y = gdk_screen_height () - widget->allocation.height;
+
+	gtk_widget_set_uposition (widget, x, y);
+}
+
+typedef struct _CursorOffset {
+	int x;
+	int y;
+} CursorOffset;
+
+static gboolean
+gdm_login_icon_released (GtkWidget *widget)
+{
+	gtk_grab_remove (widget);
+	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+
+	gtk_object_remove_data (GTK_OBJECT (widget), "offset");
+
+	/* no longer send events from this window to widget */
+	gdk_window_set_user_data (rootwin_overlay, NULL);
+
+	return TRUE;
+}
+
+static gboolean
 gdm_login_icon_pressed (GtkWidget *widget, GdkEventButton *event)
 {
     CursorOffset *p;
+    GdkCursor *fleur_cursor;
 
-    if (!widget || !event)
-	return;
-    
     if (event->type == GDK_2BUTTON_PRESS) {
-	gtk_widget_destroy (GTK_WIDGET (win));
-	gdk_window_show (login->window);
-	return;
+	    gdm_login_icon_released (widget);
+	    gtk_widget_destroy (GTK_WIDGET (win));
+	    gtk_widget_show (login);
+	    gtk_widget_grab_focus (entry);	
+	    return TRUE;
     }
     
     if (event->type != GDK_BUTTON_PRESS)
-	return;
+	return FALSE;
     
-    p = gtk_object_get_user_data (GTK_OBJECT (widget));
+
+    p = g_new0 (CursorOffset, 1);
+    gtk_object_set_data_full (GTK_OBJECT (widget), "offset", p,
+			      (GtkDestroyNotify)g_free);
     p->x = (gint) event->x;
     p->y = (gint) event->y;
-    
+
     gtk_grab_add (widget);
-    gdk_pointer_grab (widget->window, TRUE,
+    fleur_cursor = gdk_cursor_new (GDK_FLEUR);
+    gdk_pointer_grab (rootwin_overlay, TRUE,
 		      GDK_BUTTON_RELEASE_MASK |
 		      GDK_BUTTON_MOTION_MASK |
 		      GDK_POINTER_MOTION_HINT_MASK,
-		      NULL, NULL, 0);
+		      NULL,
+		      fleur_cursor,
+		      GDK_CURRENT_TIME);
+    gdk_cursor_destroy (fleur_cursor);
+    /* send events from this window to widget */
+    gdk_window_set_user_data (rootwin_overlay, widget);
+    gdk_flush ();
+
+    return TRUE;
 }
 
-
-static void
-gdm_login_icon_released (GtkWidget *widget)
-{
-    if(!widget)
-	return;
-
-    gtk_grab_remove (widget);
-    gdk_pointer_ungrab (0);
-}
-
-
-static void
-gdm_login_icon_motion (GtkWidget *widget, GdkEventMotion *event)
-{
-    gint xp, yp;
-    CursorOffset *p;
-    GdkModifierType mask;
-    
-    if(!widget || !event)
-	return;
-
-    p = gtk_object_get_user_data (GTK_OBJECT (widget));
-    gdk_window_get_pointer (rootwin, &xp, &yp, &mask);
-    gtk_widget_set_uposition (GTK_WIDGET (widget), xp-p->x, yp-p->y);
-}
-
-
 static gboolean
+gdm_login_icon_motion (GtkWidget *widget, GdkEventMotion *event)
+{
+	gint xp, yp;
+	CursorOffset *p;
+	GdkModifierType mask;
+
+	p = gtk_object_get_data (GTK_OBJECT (widget), "offset");
+
+	if (p == NULL)
+		return FALSE;
+
+	gdk_window_get_pointer (rootwin, &xp, &yp, &mask);
+	set_screen_pos (GTK_WIDGET (widget), xp-p->x, yp-p->y);
+
+	return TRUE;
+}
+
+
+static void
 gdm_login_iconify_handler (GtkWidget *widget, gpointer data)
 {
     GtkWidget *fixed;
     GtkWidget *icon;
     GdkGC *gc;
     GtkStyle *style;
-    CursorOffset *icon_pos;
     gint rw, rh, iw, ih;
 
-    gdk_window_hide (login->window);
+    gtk_widget_hide (login);
     style = gtk_widget_get_default_style();
     gc = style->black_gc; 
     win = gtk_window_new (GTK_WINDOW_POPUP);
@@ -222,17 +263,12 @@ gdm_login_iconify_handler (GtkWidget *widget, gpointer data)
     gtk_signal_connect (GTK_OBJECT (win), "motion_notify_event",
 			GTK_SIGNAL_FUNC (gdm_login_icon_motion),NULL);
 
-    icon_pos = g_new (CursorOffset, 1);
-    gtk_object_set_user_data (GTK_OBJECT (win), icon_pos);
-
     gtk_widget_show (GTK_WIDGET (win));
 
     rw = gdk_screen_width();
     rh = gdk_screen_height();
 
-    gtk_widget_set_uposition (GTK_WIDGET (win), rw-iw, rh-ih);
-
-    return(TRUE);
+    set_screen_pos (GTK_WIDGET (win), rw-iw, rh-ih);
 }
 
 
@@ -261,28 +297,29 @@ gdm_login_abort (const gchar *format, ...)
 
 /* I *really* need to rewrite this crap */
 static gchar *
-gdm_parse_enriched_string (gchar *s)
+gdm_parse_enriched_string (const gchar *s)
 {
-    gchar cmd, *buffer, *start;
+    gchar cmd, *buffer;
     gchar hostbuf[256];
     gchar *hostname, *temp1, *temp2, *display;
     struct utsname name;
+    GString *str;
 
-    if (!s)
+    if (s == NULL)
 	return(NULL);
 
     display = g_strdup (g_getenv ("DISPLAY"));
 
-    if(!display)
+    if (display == NULL)
 	return(NULL);
 
     temp1 = strchr (display, '.');
     temp2 = strchr (display, ':');
 
-    if (temp1) {
-	*temp1 = '\0';
-    } else if (temp2) {
-	*temp2 = '\0';
+    if (temp1 != NULL) {
+	    *temp1 = '\0';
+    } else if (temp2 != NULL) {
+	    *temp2 = '\0';
     } else {
 	    g_free (display);
 	    return (NULL);
@@ -291,7 +328,7 @@ gdm_parse_enriched_string (gchar *s)
     gethostname (hostbuf, 255);
     hostname = g_strdup (hostbuf);
     
-    if (!hostname) 
+    if (hostname != NULL) 
 	hostname = g_strdup ("Gnome");
 
     uname (&name);
@@ -302,72 +339,65 @@ gdm_parse_enriched_string (gchar *s)
 	return (g_strdup_printf (_("Welcome to %s"), hostname));
     }
 
-    if (!(buffer = g_malloc (4096))) {
-	syslog (LOG_ERR, _("gdm_parse_enriched_string: Could not malloc temporary buffer!"));
-	g_free (display);
-	return (NULL);
-    }
+    str = g_string_new (NULL);
 
-    start = buffer;
+    while (*s != '\0') {
 
-    while (*s) {
+	if (*s == '%' && s[1] != 0) {
+		cmd = s[1];
+		s+=2;
 
-	if (*s=='%' && (cmd = s[1]) != 0) {
-	    s+=2;
+		switch (cmd) {
 
-	    switch (cmd) {
+		case 'h': 
+			g_string_append (str, hostname);
+			break;
 
-	    case 'h': 
-		memcpy (buffer, hostname, strlen (hostname));
-		buffer += strlen (hostname);
-		break;
-		
-	    case 'n':
-	        memcpy (buffer, name.nodename, strlen (name.nodename));
-		buffer += strlen (name.nodename);
-		break;
+		case 'n':
+			g_string_append (str, name.nodename);
+			break;
 
-	    case 'd': 
-		memcpy (buffer, display, strlen (display));
-		buffer += strlen (display);
-		break;
+		case 'd': 
+			g_string_append (str, display);
+			break;
 
-	    case 's':
-	        memcpy (buffer, name.sysname, strlen (name.sysname));
-		buffer += strlen (name.sysname);
-		break;
-		
-	    case 'r':
-	        memcpy (buffer, name.release, strlen (name.release));
-	        buffer += strlen (name.release);
-	        break;
+		case 's':
+			g_string_append (str, name.sysname);
+			break;
 
-	    case 'm':
-	        memcpy (buffer, name.machine, strlen (name.machine));
-	        buffer += strlen (name.machine);
-	        break;
+		case 'r':
+			g_string_append (str, name.release);
+			break;
 
-	    case '%':
-		*buffer++ = '%';
-		break;
-		
-	    default:
-		break;
-	    };
+		case 'm':
+			g_string_append (str, name.machine);
+			break;
+
+		case '%':
+			g_string_append_c (str, '%');
+			break;
+
+		default:
+			break;
+		};
+	} else {
+		g_string_append_c (str, *s);
 	}
-	else
-	    *buffer++ = *s++;
+	s++;
     }
 
-    *buffer = 0;
-    g_free (display);
+    buffer = str->str;
+    g_string_free (str, FALSE);
 
-    return (g_strdup (start));
+    g_free (display);
+    g_free (hostname);
+
+    return buffer;
 }
 
 
 static gboolean
-gdm_login_query (gchar *msg)
+gdm_login_query (const gchar *msg)
 {
     GtkWidget *req;
 
@@ -423,12 +453,12 @@ gdm_login_parse_config (void)
 
     gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
 
-    GdmBrowser = gnome_config_get_int (GDM_KEY_BROWSER);
+    GdmBrowser = gnome_config_get_bool (GDM_KEY_BROWSER);
     GdmLogo = gnome_config_get_string (GDM_KEY_LOGO);
     GdmFont = gnome_config_get_string (GDM_KEY_FONT);
     GdmIcon = gnome_config_get_string (GDM_KEY_ICON);
-    GdmQuiver = gnome_config_get_int (GDM_KEY_QUIVER);
-    GdmSystemMenu = gnome_config_get_int (GDM_KEY_SYSMENU);
+    GdmQuiver = gnome_config_get_bool (GDM_KEY_QUIVER);
+    GdmSystemMenu = gnome_config_get_bool (GDM_KEY_SYSMENU);
     GdmUserMaxFile = gnome_config_get_int (GDM_KEY_MAXFILE);
     GdmRelaxPerms = gnome_config_get_int (GDM_KEY_RELAXPERM);
     GdmLocaleFile = gnome_config_get_string (GDM_KEY_LOCFILE);
@@ -440,11 +470,13 @@ gdm_login_parse_config (void)
     GdmExclude = gnome_config_get_string (GDM_KEY_EXCLUDE);
     GdmGlobalFaceDir = gnome_config_get_string (GDM_KEY_FACEDIR);
     GdmDefaultFace = gnome_config_get_string (GDM_KEY_FACE);
-    GdmDebug = gnome_config_get_int (GDM_KEY_DEBUG);
-    GdmUserMaxFile = gnome_config_get_int (GDM_KEY_MAXFILE);
-    GdmRelaxPerms = gnome_config_get_int (GDM_KEY_RELAXPERM);
+    GdmDebug = gnome_config_get_bool (GDM_KEY_DEBUG);
     GdmIconMaxWidth = gnome_config_get_int (GDM_KEY_ICONWIDTH);
     GdmIconMaxHeight = gnome_config_get_int (GDM_KEY_ICONHEIGHT);
+    GdmLockPosition = gnome_config_get_bool (GDM_KEY_LOCK_POSITION);
+    GdmSetPosition = gnome_config_get_bool (GDM_KEY_SET_POSITION);
+    GdmPositionX = gnome_config_get_int (GDM_KEY_LOCK_POSITION);
+    GdmPositionY = gnome_config_get_int (GDM_KEY_SET_POSITION);
 
     gnome_config_pop_prefix();
 
@@ -453,18 +485,18 @@ gdm_login_parse_config (void)
 
 
     /* Disable System menu on non-local displays */
-    display = getenv ("DISPLAY");
+    display = g_getenv ("DISPLAY");
 
     if (!display)
 	gdm_login_abort ("gdm_login_parse_config: DISPLAY variable not set!");
 
     if (strncmp (display, ":", 1))
-	GdmSystemMenu = 0;
+	GdmSystemMenu = FALSE;
 }
 
 
 static gboolean 
-gdm_login_list_lookup (GSList *l, gchar *data)
+gdm_login_list_lookup (GSList *l, const gchar *data)
 {
     GSList *list = l;
 
@@ -473,7 +505,7 @@ gdm_login_list_lookup (GSList *l, gchar *data)
 
     while (list) {
 
-	if (!strcasecmp (list->data, data))
+	if (g_strcasecmp (list->data, data) == 0)
 	    return (TRUE);
 	
 	list = list->next;
@@ -484,9 +516,9 @@ gdm_login_list_lookup (GSList *l, gchar *data)
 
 
 static void
-gdm_login_session_lookup (gchar* savedsess)
+gdm_login_session_lookup (const gchar* savedsess)
 {
-    if (!curuser)
+    if (curuser == NULL)
 	gdm_login_abort("gdm_login_session_lookup: curuser==NULL. Mail <mkp@mkp.net> with " \
 			"information on your PAM and user database setup");
 
@@ -499,25 +531,29 @@ gdm_login_session_lookup (gchar* savedsess)
     if ( ! (savedsess != NULL &&
 	    strcmp ("(null)", savedsess) != 0 &&
 	    savedsess[0] != '\0')) {
-	/* If "Last" is chosen run Default, else run user's current selection */
-	if (!strcasecmp (cursess, lastsess))
-	    session = defsess;
-	else
-	    session = cursess;
+	    /* If "Last" is chosen run Default,
+	     * else run user's current selection */
+	    g_free (session);
+	    if (g_strcasecmp (cursess, lastsess) == 0)
+		    session = g_strdup (defsess);
+	    else
+		    session = g_strdup (cursess);
 
-	savesess = TRUE;
-	return;
+	    savesess = TRUE;
+	    return;
     }
 
     /* If "Last" session is selected */
-    if (!strcasecmp (cursess, lastsess)) { 
-	session = savedsess;
+    if (g_strcasecmp (cursess, lastsess) == 0) { 
+	g_free (session);
+	session = g_strdup (savedsess);
 
 	/* Check if user's saved session exists on this box */
 	if (!gdm_login_list_lookup (sessions, session)) {
 	    gchar *msg;
 
-	    session = defsess;
+	    g_free (session);
+	    session = g_strdup (defsess);
 	    msg = g_strdup_printf (_("Your preferred session type %s is not installed on this machine.\n" \
 				     "Do you wish to make %s the default for future sessions?"),
 				   savedsess, defsess);	    
@@ -527,10 +563,11 @@ gdm_login_session_lookup (gchar* savedsess)
     }
     /* One of the other available session types is selected */
     else { 
-	session = cursess;
+	g_free (session);
+	session = g_strdup (cursess);
 
 	/* User's saved session is not the chosen one */
-	if (strcasecmp (savedsess, session)) {
+	if (g_strcasecmp (savedsess, session) != 0) {
 	    gchar *msg;
 
 	    msg = g_strdup_printf (_("You have chosen %s for this session, but your default setting is " \
@@ -544,9 +581,9 @@ gdm_login_session_lookup (gchar* savedsess)
 
 
 static void
-gdm_login_language_lookup (gchar* savedlang)
+gdm_login_language_lookup (const gchar* savedlang)
 {
-    if (!curuser)
+    if (curuser == NULL)
 	gdm_login_abort("gdm_login_language_lookup: curuser==NULL. Mail <mkp@mkp.net> with " \
 			"information on your PAM and user database setup");
 
@@ -558,21 +595,23 @@ gdm_login_language_lookup (gchar* savedlang)
     /* Previously saved language not found in ~user/.gnome/gdm */
     if (savedlang && ! strlen (savedlang)) {
 	/* If "Last" is chosen use Default, else use current selection */
-	if (!strcasecmp (curlang, lastlang))
-	    language = GdmDefaultLocale;
+	g_free (language);
+	if (g_strcasecmp (curlang, lastlang) == 0)
+	    language = g_strdup (GdmDefaultLocale);
 	else
-	    language = curlang;
+	    language = g_strdup (curlang);
 
 	savelang = TRUE;
 	return;
     }
 
     /* If a different language is selected */
-    if (strcasecmp (curlang, lastlang)) {
-	language = curlang;
+    if (g_strcasecmp (curlang, lastlang) != 0) {
+        g_free (language);
+	language = g_strdup (curlang);
 
 	/* User's saved language is not the chosen one */
-	if (strcasecmp (savedlang, language)) {
+	if (g_strcasecmp (savedlang, language) != 0) {
 	    gchar *msg;
 
 	    msg = g_strdup_printf (_("You have chosen %s for this session, but your default setting is " \
@@ -581,14 +620,61 @@ gdm_login_language_lookup (gchar* savedlang)
 	    savelang = gdm_login_query (msg);
 	    g_free (msg);
 	}
+    } else {
+	g_free (language);
+	language = g_strdup (savedlang);
     }
-    else
-	language = savedlang;
 
     /* Now this is utterly ugly, but I suppose it works */
     language[0] = tolower (language[0]);
 }
 
+static int dance_handler = 0;
+
+static gboolean
+dance (gpointer data)
+{
+	static double t1 = 0.0, t2 = 0.0;
+	double xm, ym;
+	int x, y;
+	static int width = -1;
+	static int height = -1;
+
+	if (width == -1)
+		width = gdk_screen_width ();
+	if (height == -1)
+		height = gdk_screen_height ();
+
+	if (login == NULL ||
+	    login->window == NULL) {
+		dance_handler = 0;
+		return FALSE;
+	}
+
+	xm = cos (1.01 * t1);
+	ym = sin (0.90 * t2);
+
+	t1 += 0.03 + (rand () % 10) / 500.0;
+	t2 += 0.03 + (rand () % 10) / 500.0;
+
+	x = (width / 2) + (width / 5) * xm;
+	y = (height / 2) + (height / 5) * ym;
+
+	set_screen_pos (login,
+			x - login->allocation.width / 2,
+			y - login->allocation.height / 2);
+
+	return TRUE;
+}
+
+static void
+evil (const char *user)
+{
+	if (dance_handler == 0 &&
+	    strcmp (user, "Start Dancing") == 0) {
+		dance_handler = gtk_timeout_add (50, dance, NULL);
+	}
+}
 
 static gboolean
 gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
@@ -599,7 +685,7 @@ gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
     switch (event->keyval) {
 
     case GDK_Return:
-	gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
+	gtk_widget_set_sensitive (entry, FALSE);
 
 	if (GdmBrowser)
 	    gtk_widget_set_sensitive (GTK_WIDGET (browser), FALSE);
@@ -609,9 +695,13 @@ gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 	 * setups. Needs thinking! 
 	 */
 
-	if (!curuser)
+	if (curuser == NULL) {
 	    curuser = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
+	    /* evilness */
+	    evil (curuser);
+	}
+
 	g_print ("%c%s\n", STX, gtk_entry_get_text (GTK_ENTRY (entry)));
 	break;
 
@@ -621,8 +711,6 @@ gdm_login_entry_handler (GtkWidget *widget, GdkEventKey *event)
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (entry), "key_press_event");
 	break;
 
-	break;
-
     default:
 	break;
     }
@@ -659,7 +747,9 @@ gdm_login_session_init (GtkWidget *menu)
     item = gtk_radio_menu_item_new_with_label (NULL, lastsess);
     sessgrp = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (item));
     gtk_menu_append (GTK_MENU (menu), item);
-    gtk_signal_connect (GTK_OBJECT (item), "activate", gdm_login_session_handler, NULL);
+    gtk_signal_connect (GTK_OBJECT (item), "activate",
+			GTK_SIGNAL_FUNC (gdm_login_session_handler),
+			NULL);
     gtk_widget_show (GTK_WIDGET (item));
 
     item = gtk_menu_item_new();
@@ -687,7 +777,8 @@ gdm_login_session_init (GtkWidget *menu)
 	lstat (s, &statbuf);
 
 	/* If default session link exists, find out what it points to */
-	if (S_ISLNK (statbuf.st_mode) && !strcasecmp (dent->d_name, "default")) {
+	if (S_ISLNK (statbuf.st_mode) &&
+	    g_strcasecmp (dent->d_name, "default") == 0) {
 	    gchar t[_POSIX_PATH_MAX];
 	    
 	    linklen = readlink (s, t, _POSIX_PATH_MAX);
@@ -707,7 +798,8 @@ gdm_login_session_init (GtkWidget *menu)
 		sessions = g_slist_append (sessions, dent->d_name);
 		gtk_menu_append (GTK_MENU (menu), item);
 		gtk_signal_connect (GTK_OBJECT (item), "activate",
-				    GTK_SIGNAL_FUNC (gdm_login_session_handler), NULL);
+				    GTK_SIGNAL_FUNC (gdm_login_session_handler),
+				    NULL);
 		gtk_widget_show (GTK_WIDGET (item));
 	    }
 	    else 
@@ -878,9 +970,9 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	gtk_widget_show (GTK_WIDGET (label));
 	gtk_entry_set_text (GTK_ENTRY (entry), "");
 	gtk_entry_set_visibility (GTK_ENTRY (entry), TRUE);
-	gtk_widget_set_sensitive (GTK_WIDGET (entry), TRUE);
-	gtk_widget_grab_focus (GTK_WIDGET (entry));	
-	gtk_widget_show (GTK_WIDGET (entry));
+	gtk_widget_set_sensitive (entry, TRUE);
+	gtk_widget_grab_focus (entry);	
+	gtk_widget_show (entry);
 	break;
 
     case GDM_NOECHO:
@@ -899,9 +991,9 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	gtk_widget_show (GTK_WIDGET (label));
 	gtk_entry_set_text (GTK_ENTRY (entry), "");
 	gtk_entry_set_visibility (GTK_ENTRY (entry), FALSE);
-	gtk_widget_set_sensitive (GTK_WIDGET (entry), TRUE);
-	gtk_widget_grab_focus (GTK_WIDGET (entry));	
-	gtk_widget_show (GTK_WIDGET (entry));
+	gtk_widget_set_sensitive (entry, TRUE);
+	gtk_widget_grab_focus (entry);	
+	gtk_widget_show (entry);
 	break;
 
     case GDM_MSGERR:
@@ -960,12 +1052,12 @@ gdm_login_ctrl_handler (GIOChannel *source, GIOCondition cond, gint fd)
 	    }
 	}
 
-	if (curuser) {
+	if (curuser != NULL) {
 	    g_free (curuser);
-	    curuser=NULL;
+	    curuser = NULL;
 	}
 
-	gtk_widget_set_sensitive (GTK_WIDGET (entry), TRUE);
+	gtk_widget_set_sensitive (entry, TRUE);
 
 	if (GdmBrowser)
 	    gtk_widget_set_sensitive (GTK_WIDGET (browser), TRUE);
@@ -1029,10 +1121,10 @@ gdm_login_browser_select (GtkWidget *widget, gint selected, GdkEvent *event)
 	if (user && user->login)
 	    gtk_entry_set_text (GTK_ENTRY (entry), user->login);
 
-	if (!curuser)
+	if (curuser == NULL)
 	    curuser = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
-	gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
+	gtk_widget_set_sensitive (entry, FALSE);
 	gtk_widget_set_sensitive (GTK_WIDGET (browser), FALSE);
 	g_print ("%c%s\n", STX, gtk_entry_get_text (GTK_ENTRY (entry)));
 	break;
@@ -1122,15 +1214,197 @@ gdm_init_root_window_overlay (void)
 	gdk_window_add_filter (rootwin_overlay, root_keys_filter, NULL);
 }
 
+static gboolean
+handle_expose (GtkWidget *handle, GdkEventExpose *event, gpointer data)
+{
+	if (handle->window != NULL)
+		gtk_paint_handle (handle->style,
+				  handle->window,
+				  handle->state,
+				  GTK_SHADOW_NONE,
+				  &event->area,
+				  handle,
+				  "gdm-handle",
+				  0, 0,
+				  handle->allocation.width,
+				  handle->allocation.height,
+				  GTK_ORIENTATION_HORIZONTAL);
+	return TRUE;
+}
+
+static gboolean
+gdm_login_handle_pressed (GtkWidget *widget, GdkEventButton *event)
+{
+    gint xp, yp;
+    GdkModifierType mask;
+    CursorOffset *p;
+    GdkCursor *fleur_cursor;
+
+    if (login == NULL ||
+	login->window == NULL ||
+	event->type != GDK_BUTTON_PRESS ||
+	GdmLockPosition)
+	    return FALSE;
+
+    p = g_new0 (CursorOffset, 1);
+    gtk_object_set_data_full (GTK_OBJECT (widget), "offset", p,
+			      (GtkDestroyNotify)g_free);
+    
+    gdk_window_get_pointer (login->window, &xp, &yp, &mask);
+    p->x = xp;
+    p->y = yp;
+
+    gtk_grab_add (widget);
+    fleur_cursor = gdk_cursor_new (GDK_FLEUR);
+    gdk_pointer_grab (rootwin_overlay, TRUE,
+		      GDK_BUTTON_RELEASE_MASK |
+		      GDK_BUTTON_MOTION_MASK |
+		      GDK_POINTER_MOTION_HINT_MASK,
+		      NULL,
+		      fleur_cursor,
+		      GDK_CURRENT_TIME);
+    gdk_cursor_destroy (fleur_cursor);
+    /* send events from this window to widget */
+    gdk_window_set_user_data (rootwin_overlay, widget);
+    gdk_flush ();
+    
+    return TRUE;
+}
+
+static gboolean
+gdm_login_handle_released (GtkWidget *widget, GdkEventButton *event)
+{
+	gtk_grab_remove (widget);
+	gdk_pointer_ungrab (GDK_CURRENT_TIME);
+
+	gtk_object_remove_data (GTK_OBJECT (widget), "offset");
+
+	/* no longer send events from this window to widget */
+	gdk_window_set_user_data (rootwin_overlay, NULL);
+
+	return TRUE;
+}
+
+
+static gboolean
+gdm_login_handle_motion (GtkWidget *widget, GdkEventMotion *event)
+{
+    gint xp, yp;
+    CursorOffset *p;
+    GdkModifierType mask;
+
+    p = gtk_object_get_data (GTK_OBJECT (widget), "offset");
+
+    if (p == NULL)
+	    return FALSE;
+
+    gdk_window_get_pointer (rootwin, &xp, &yp, &mask);
+    set_screen_pos (GTK_WIDGET (login), xp-p->x, yp-p->y);
+
+    GdmSetPosition = TRUE;
+    GdmPositionX = xp - p->x;
+    GdmPositionY = yp - p->y;
+    gtk_window_position (GTK_WINDOW (login), GTK_WIN_POS_NONE);
+
+    return TRUE;
+}
+
+
+static GtkWidget *
+create_handle (void)
+{
+	GtkWidget *ebox, *hbox, *frame, *w;
+
+	ebox = gtk_event_box_new ();
+	gtk_signal_connect (GTK_OBJECT (ebox), "button_press_event",
+			    GTK_SIGNAL_FUNC (gdm_login_handle_pressed),
+			    NULL);
+	gtk_signal_connect (GTK_OBJECT (ebox), "button_release_event",
+			    GTK_SIGNAL_FUNC (gdm_login_handle_released),
+			    NULL);
+	gtk_signal_connect (GTK_OBJECT (ebox), "motion_notify_event",
+			    GTK_SIGNAL_FUNC (gdm_login_handle_motion),
+			    NULL);
+
+	frame = gtk_frame_new (NULL);
+	gtk_container_add (GTK_CONTAINER (ebox), frame);
+	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (frame), hbox);
+
+	w = gtk_drawing_area_new ();
+	gtk_signal_connect (GTK_OBJECT (w), "expose_event",
+			    GTK_SIGNAL_FUNC (handle_expose),
+			    NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), w, TRUE, TRUE, 0);
+
+	w = gtk_label_new (_("GNOME Desktop Manager"));
+	gtk_misc_set_padding (GTK_MISC (w), GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, GNOME_PAD_SMALL);
+	
+	w = gtk_drawing_area_new ();
+	gtk_signal_connect (GTK_OBJECT (w), "expose_event",
+			    GTK_SIGNAL_FUNC (handle_expose),
+			    NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), w, TRUE, TRUE, 0);
+
+	if (GdmIcon != NULL) {
+		if (access (GdmIcon, R_OK)) {
+			syslog (LOG_WARNING, _("Can't open icon file: %s. Suspending iconify feature!"), GdmIcon);
+		} else {
+			w = gtk_button_new ();
+			gtk_container_add (GTK_CONTAINER (w),
+					   gtk_arrow_new (GTK_ARROW_DOWN,
+							  GTK_SHADOW_OUT));
+			gtk_signal_connect
+				(GTK_OBJECT (w), "clicked",
+				 GTK_SIGNAL_FUNC (gdm_login_iconify_handler), 
+				 NULL);
+			GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_FOCUS);
+			GTK_WIDGET_UNSET_FLAGS (w, GTK_CAN_DEFAULT);
+			gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
+		}
+	}
+
+	gtk_widget_show_all (ebox);
+
+	return ebox;
+}
+
+static void
+gdm_login_set_font (GtkWidget *widget, const char *font_name)
+{
+	GdkFont *font;
+	GtkStyle *new_style;
+	
+	g_return_if_fail (widget != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+	g_return_if_fail (font_name != NULL);
+
+	font = gdk_fontset_load (font_name);
+
+	if (font == NULL)
+		return;
+	
+	gtk_widget_set_rc_style (widget);
+	new_style = gtk_style_copy (gtk_widget_get_style (widget));
+
+	gdk_font_unref (new_style->font);
+	new_style->font = font;
+	
+	gtk_widget_set_style (widget, new_style);
+	gtk_style_unref (new_style);
+}
+
 static void
 gdm_login_gui_init (void)
 {
     GtkWidget *frame1, *frame2;
     GtkWidget *mbox, *menu, *menubar, *item, *welcome;
-    GtkWidget *table, *stack, *hline1, *hline2;
+    GtkWidget *table, *stack, *hline1, *hline2, *handle;
     GtkWidget *bbox = NULL;
     GtkWidget *logoframe = NULL;
-    GtkStyle *style;
     gchar *greeting;
     gint cols, rows;
     struct stat statbuf;
@@ -1173,6 +1447,9 @@ gdm_login_gui_init (void)
     gtk_widget_show (mbox);
     gtk_container_add (GTK_CONTAINER (frame2), mbox);
 
+    handle = create_handle ();
+    gtk_box_pack_start (GTK_BOX (mbox), handle, FALSE, FALSE, 0);
+
     menubar = gtk_menu_bar_new();
     gtk_widget_ref (GTK_WIDGET (menubar));
     gtk_box_pack_start (GTK_BOX (mbox), menubar, FALSE, FALSE, 0);
@@ -1213,20 +1490,6 @@ gdm_login_gui_init (void)
 	gtk_widget_show (GTK_WIDGET (item));
     }
 
-    if (GdmIcon) {
-	if (access (GdmIcon, R_OK)) {
-	    syslog (LOG_WARNING, _("Can't open icon file: %s. Suspending iconify feature!"), GdmIcon);
-	}
-	else {
-	    item = gtk_menu_item_new_with_label (_("Iconify"));
-	    gtk_menu_bar_append (GTK_MENU_BAR (menubar), item);
-	    gtk_signal_connect (GTK_OBJECT (item), "activate",
-			       GTK_SIGNAL_FUNC (gdm_login_iconify_handler), 
-				NULL);
-	    gtk_widget_show (GTK_WIDGET (item));
-	}
-    }
-
     if (GdmBrowser)
 	rows = 2;
     else
@@ -1317,17 +1580,10 @@ gdm_login_gui_init (void)
 			      (GtkDestroyNotify) gtk_widget_unref);
     gtk_widget_show (stack);
 
-    style = gtk_style_new();
-    gdk_font_unref (style->font);
-    style->font = gdk_font_load (GdmFont);
-
-    if (style->font)
-	gtk_widget_push_style (style);
-
     greeting = gdm_parse_enriched_string (GdmWelcome);    
     welcome = gtk_label_new (greeting);
-    gtk_widget_set_name(welcome, "Welcome");
-    g_free(greeting);
+    gtk_widget_set_name (welcome, "Welcome");
+    g_free (greeting);
     gtk_widget_ref (welcome);
     gtk_object_set_data_full (GTK_OBJECT (login), "welcome", welcome,
 			      (GtkDestroyNotify) gtk_widget_unref);
@@ -1335,8 +1591,8 @@ gdm_login_gui_init (void)
     gtk_table_attach (GTK_TABLE (stack), welcome, 0, 1, 0, 1,
 		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 		      (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-
-    gtk_widget_pop_style ();
+    if (GdmFont != NULL)
+	    gdm_login_set_font (welcome, GdmFont);
 
     hline1 = gtk_hseparator_new ();
     gtk_widget_ref (hline1);
@@ -1427,10 +1683,15 @@ gdm_login_gui_init (void)
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 			  (GtkAttachOptions) (GTK_FILL), 0, 0);
     
-    gtk_widget_grab_focus (GTK_WIDGET (entry));	
+    gtk_widget_grab_focus (entry);	
     gtk_window_set_focus (GTK_WINDOW (login), entry);	
     gtk_window_set_policy (GTK_WINDOW (login), 1, 1, 1);
-    gtk_window_position (GTK_WINDOW (login), GTK_WIN_POS_CENTER);
+    
+    if (GdmSetPosition) {
+	    set_screen_pos (login, GdmPositionX, GdmPositionY);
+    } else {
+	    gtk_window_position (GTK_WINDOW (login), GTK_WIN_POS_CENTER);
+    }
 
     gtk_widget_show_all (GTK_WIDGET (login));
 }
@@ -1525,11 +1786,11 @@ gdm_login_check_exclude (struct passwd *pwent)
     gint i;
 
     for (i=0 ; lockout_passes[i] ; i++) 
-	if(! strcmp (lockout_passes[i], pwent->pw_passwd))
+	if(strcmp (lockout_passes[i], pwent->pw_passwd) == 0)
 	    return (TRUE);
  
      while (list && list->data) {
-	 if (! strcasecmp (pwent->pw_name, (gchar *) list->data))
+	 if (g_strcasecmp (pwent->pw_name, (gchar *) list->data) == 0)
 	     return (TRUE);
 
 	 list = list->next;
@@ -1565,7 +1826,7 @@ gdm_login_users_init (void)
 
     if (access (GdmDefaultFace, R_OK)) {
 	syslog (LOG_WARNING, _("Can't open DefaultImage: %s. Suspending face browser!"), GdmDefaultFace);
-	GdmBrowser = 0;
+	GdmBrowser = FALSE;
 	return;
     }
     else 
